<?php

//require_once('../magpierss/rss_fetch.inc');
//require_once('../magpierss/rss_utils.inc');

//----------------------------------------------------------------------------
function readRssFeeds (&$db, &$user, &$category)
{
  foreach (feed::all($db, $user->userid,
    ($category ? "WHERE cat_id='$category->id' " : '') .'ORDER BY name') as $feed)
  {
      readRssFeed ($db, $user, $feed);
  }
}

//----------------------------------------------------------------------------
function readRssFeed (&$db, &$user, &$feed)
{
  global $DEBUG;
  $last_update = strtotime($feed->last_update);

  // Only fetch the RSS feed so often (it may also be cached by Magpie)
  if ((time() - $last_update) > $user->timeout) {

    if ($DEBUG) { echo '<!-- fetching sources for '. $feed->rss_link ." -->\n"; }

    if (!$DEBUG) error_reporting(E_USER_ERROR);  // suppress noisy magpie warnings
    $rss = fetch_rss($feed->rss_link);

    if ($rss && !isset($rss->from_cache)) {

      if (empty($feed->image_link) && $user->show_images && $rss->image)
        saveFeedImage ($db, $user, $feed, $rss);

      // Reverse the array so that we add on oldest first; this preserves the assumption
      // that newer DB ids are newer articles.
      $items = array_reverse($rss->items);

      foreach ($items as $item) {

        $link = linkFromItem ($item, "{$user->userid}{$feed->id}");

        if ($link->insert($db)) {
          if ($DEBUG) { echo "<!-- inserted link {$link->id} -->\n"; }
        } else {
          if ($DEBUG) { echo '<!-- '. $db->error() ." -->\n"; } // 'Duplicate entry' is normal
        }
      }

    } else {

      if (!isset($rss->from_cache))
        echo '<p class="error">Could not fetch sources for '. htmlspecialchars($feed->name) .': '. magpie_error() . "</p>\n";
    }

    // now we nuke the old rows
    $links = feedlink::all ($db, "WHERE src_id='". $user->userid . $feed->id .
      "' ORDER BY pubdate DESC, id DESC LIMIT ". $user->max_links . ',1');
    if (count($links) > 0) {
      $cutRow = $links[0]->id;

      if ($DEBUG) { echo  "<!-- cut row is $cutRow -->\n"; }
      
      // XXX move sql to feedlink class
      if (!$db->query('DELETE FROM '. DB_PREFIX ."links WHERE id < $cutRow AND src_id='{$user->userid}{$feed->id}'")) {
        if ($DEBUG) { echo  "<!-- delete old links failed -->\n"; }
      }
    }

    // now we freshen the last_update time
    $feed->last_update = date('Y-m-d H:i:s');
    if (!$feed->update ($db, $user->userid)) {
      if ($DEBUG) { echo '<!-- last_update update failed -->'; }
    }
  }
}

function linkFromItem ($item, $srcid)
{
  // Pull the date
  if (isset($item['date_timestamp'])) {
    $pubd = date("Y-m-d H:i:s", $item['date_timestamp']);
  } else if (isset($item['pubdate'])) {
    $pubd = date("Y-m-d H:i:s", strtotime($item['pubdate']));
  } else {
    $pubd = date("Y-m-d H:i:s", time());
  }

  // Make up a title if one doesn't exist
  if (!isset($item['title']) || empty($item['title'])) {
    $item['title'] = substr ($item['description'], 0, 40);
  }

  // use guid field if link is empty
  if (!isset($item['link']) || empty($item['link'])) {
    $item['link'] = $item['guid'];
  }

  // Favor atom_content, since it often has the entire contents
  if (isset($item['atom_content'])) {
    $item['description'] = $item['atom_content'];
  } else if (!isset($item['description'])) {
    $item['description'] = '';
  }

  $link = new feedlink();
  $link->src_id = $srcid;
  $link->link = $item['link']; // XXX make sure it looks okay
  $link->title = strip_tags_xss (XSS_TAGS_NONE, $item['title']);
  $link->description = strip_tags_xss (XSS_TAGS, $item['description']);
  $link->state = 'new';
  $link->pubdate = $pubd;

  return $link;
}

//------------------
// Purge the cached file for this url.  This uses magpie internals, as it
// should really be provided there.
//
function purge_rss ($url) {
  init();

  if ( !isset($url) )
    return false;

  if ( !MAGPIE_CACHE_ON )
    return false;

  $cache = new RSSCache( MAGPIE_CACHE_DIR, MAGPIE_CACHE_AGE );

  if ($cache->ERROR)
    return false;

  $cache_key = $url . MAGPIE_OUTPUT_ENCODING;
  $filename = $cache->file_name($cache_key);

  if (file_exists($filename)) {
    unlink($filename);
  }

  return true;
}

function saveFeedImage (&$db, &$user, &$feed, &$rss)
{
  global $DEBUG;

  if ($rss->image) {
    if (FEED_IMG_CACHE) {

      $imgUrl = $rss->image['url'];
      if (($imgFile = fopen ($imgUrl, 'rb')) === FALSE) {
        if ($DEBUG) echo "\n<!-- error opening $imgUrl -->\n";
        return;
      }

      $outFileName = FEED_IMG_FOLDER .'/'. $user->userid . $feed->id . trim(strrchr($imgUrl, '.'));
      if (($outFile = fopen ($outFileName, 'wb')) === FALSE) {
        if ($DEBUG) echo "\n<!-- error opening $outFileName -->\n";
        return;
      }

      while (!feof($imgFile)) {
        $chunk = fread ($imgFile, 1024);
        fwrite ($outFile, $chunk);
      }

      fclose ($imgFile);
      fclose ($outFile);

      resizeFeedImage ($outFileName);

    } else {
      $outFileName = $rss->image['url'];
    }

    $feed->image_link = $outFileName;
    if (!$feed->update ($db, $user->userid))
      if ($DEBUG) echo "\n<!-- error writing feed $feed->id -->\n";
  }
}

function resizeFeedImage ($filename)
{
  if (FEED_IMG_CACHE && FEED_IMG_HEIGHT > 0) {
    list($width, $height, $type) = getImageSize($filename);

    if ($height > FEED_IMG_HEIGHT) {
      $scale = FEED_IMG_HEIGHT / $height;
      $newHeight = $height * $scale;
      $newWidth = $width * $scale;

      switch ($type) {
        case IMAGETYPE_JPEG:
          $origImg = @imageCreateFromJpeg ($filename);
          break;
        case IMAGETYPE_GIF:
          $origImg = @imageCreateFromGif ($filename);
          break;
        case IMAGETYPE_PNG:
          $origImg = @imageCreateFromPng ($filename);
          break;
        default:
          return;
      }

      if (!$origImg)
        return;

      $newImg = imageCreateTrueColor ($newWidth, $newHeight);

      imageCopyResampled ($newImg, $origImg, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);

      switch ($type) {
        case IMAGETYPE_JPEG:
          imageJpeg ($newImg, $filename);
          break;
        case IMAGETYPE_GIF:
          imageGif ($newImg, $filename);
          break;
        case IMAGETYPE_PNG:
          imagePng ($newImg, $filename);
          break;
        default:
          return;
      }
    }
  }
}

