<?php
/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * Installation is a 3- or 4-step process:
 *  1 - gather DB access info and options
 *  2 - create DB
 *  3 - add a user
 *  4 - select default user
 */

require_once('inc/output.php');
require_once('inc/functions.php');
require_once('inc/security.php');

$configFile = 'inc/config_user.php';
$DEBUG = 0;
if ($DEBUG) error_reporting(E_ALL);

outputHeader(NULL, 'Installation');
echo "<div id=\"wrapper\"><div id=\"content\"><div class=\"category\">\n";

// -------------------------------------------------------------------------
// Write config to the configfile
//
if (isset($_POST['writeConfig'])) {

  if (!isset($_POST['dbhost']) || !isset($_POST['dbuser']) || !isset($_POST['dbpass']) || !isset($_POST['dbname'])) {

    echo "<p>You must at least enter all the database information.</p>";

  } else {

    require_once('inc/config.php');

    if (!($f = fopen ($configFile, 'w'))) {

      echo "<p>Failed to open config file for writing.  Check that the rnews folder and $configFile file have write permissions set.</p>";

    } else {

      $dbprefix = "'". (isset($_POST['dbprefix']) ? $_POST['dbprefix'] : '') ."'";
      $title = "'". (isset($_POST['title']) ? $_POST['title'] : 'Feed Aggregator') ."'";
      $secret = "'". (isset($_POST['secret']) ? $_POST['secret'] : '') ."'";
      $forceSlogin = array_key_exists('forceSlogin', $_POST) ? 'TRUE' : 'FALSE';
      $tags = ((isset($_POST['xssStruct']) && $_POST['xssStruct']) ? XSS_TAGS_STRUCT : 0) |
              ((isset($_POST['xssFormat']) && $_POST['xssFormat']) ? XSS_TAGS_FORMAT : 0) |
              ((isset($_POST['xssImage']) && $_POST['xssImage']) ? XSS_TAGS_IMAGE : 0) |
              ((isset($_POST['xssTable']) && $_POST['xssTable']) ? XSS_TAGS_TABLE : 0) |
              ((isset($_POST['xssAny']) && $_POST['xssAny']) ? XSS_TAGS_ANY : 0);
      $cache = (isset($_POST['cache']) && $_POST['cache']) ? 'TRUE' : 'FALSE';
      $cacheImages = array_key_exists('cacheImages', $_POST) ? 'TRUE' : 'FALSE';
      $imgFolder = "'". (isset($_POST['imgFolder']) ? $_POST['imgFolder'] : '') ."'";
      $imgHeight = (isset($_POST['imgHeight']) && is_numeric($_POST['imgHeight']) && intval($_POST['imgHeight']) > 0) ? intval($_POST['imgHeight']) : 0;

      $config = "<?php
define('RNEWS_TITLE', ". $title .");
define('DB_HOST', '". $_POST['dbhost'] ."');
define('DB_USER', '". $_POST['dbuser'] ."');
define('DB_PASS', '". $_POST['dbpass'] ."');
define('DB_DATABASE', '". $_POST['dbname'] ."');
define('DB_PREFIX', ". $dbprefix .");
define('DEFAULT_USER', ". DEFAULT_USER .");
define('NEWUSER_SECRET', ". $secret .");
define('XSS_TAGS', ". $tags .");
define('FORCE_SLOGIN', ". $forceSlogin .");
define('COOKIE_HASH_KEY', '". getRandomHex(16) ."');
define('MAGPIE_CACHE_ON', ". $cache .");
define('FEED_IMG_CACHE', ". $cacheImages .");
define('FEED_IMG_FOLDER', ". $imgFolder .");
define('FEED_IMG_HEIGHT', ". $imgHeight .");
?>";

      fwrite($f, $config);
      fclose($f);
?>

  <form action="<?php echo $_SERVER['PHP_SELF'] ?>" method="POST">
    <p>Configuration was successfully updated in file <tt><?php echo $configFile; ?></tt>, click below to proceed with database creation.</p>
<?php if (isset($_POST['defaultUser']) && $_POST['defaultUser']): ?>
    <p>Then you can add user accounts and select the default user.</p>
    <input type="hidden" name="defaultUser" value="1" />
<?php endif; ?>
    <p><input type="Submit" name="makeDb" value="Create Database" /></p>
  </form>

<?php
    }
	}

  // -------------------------------------------------------------------------
  // Make the database from the afore-written config.
  //
} elseif (isset($_POST['makeDb'])) {

  require_once('inc/config.php');
  require_once('inc/db.php');
  require_once('inc/user.php');
  require_once('inc/feedlink.php');

  if (!db::create()) {

    echo '<p>Database creation failed ('. mysql_error() .'), please check the parameters and try again.</p>';

  } else {

    $db = new DB();
    $db->open() or die ("Unable to connect to database.");

    // Create user_prefs and links table
    if (!user::create($db) || !feedlink::create($db)) {
      echo '<p>Table creation failed.</p>';
    } else {
      $du = (isset($_POST['defaultUser']) && $_POST['defaultUser']);

      echo '<p>Database was initialized successfully.</p>';
      echo checkInsecureFilePerms();
      echo '<p>You can now <a href="add_user.php'.
        ($du ? '?defaultUser=1' : '') .'">create a user account</a>'.
        ($du ? ', preferably the one you will select as the default user' : '')
        .'.</p>';
    }
  }

  // -------------------------------------------------------------------------
  // Select the default user
  //
} elseif (isset($_REQUEST['selectDefaultUser'])) {
  require_once('inc/config.php');
  require_once('inc/db.php');
  require_once('inc/user.php');

  $db = new DB();
  $db->open() or die ('Unable to connect to database.');
?>
  <form action="<?php echo $_SERVER['PHP_SELF'] ?>" method="POST">
    <fieldset>
      <legend>Install Rnews (2)</legend>
      <fieldset>
        <legend>Security</legend>
        <p>By default, each user must log in to view their feeds, otherwise no feeds are visible.  However, you may wish to select one user's feeds to be shown to anyone at all without logging in.  Feeds can be added and deleted only after logging in, but anyone can mark them as read.</p>
        <p>Select the default user:
        <select name="defaultUserSelection">
        <option value="">-no user selected-</option>
<?php
  foreach (user::all($db) as $user) {
    echo '<option '. ((DEFAULT_USER == $user->id) ? 'selected="1" ' : '') 
      .' value="' . $user->id . '">'. htmlspecialchars($user->name, ENT_NOQUOTES) .'</option>';
  }
?>
        </select>
      </fieldset>
      <p><input type="Submit" name="writeDefaultUser" value="Submit"></p>
    </fieldset>
  </form>

<?php
  // -------------------------------------------------------------------------
  // Write the default user
  //
} elseif (isset($_REQUEST['writeDefaultUser'])) {

  if (isset($_POST['defaultUserSelection']) && intval($_POST['defaultUserSelection'])) {

    if ($s = file_get_contents($configFile)) {

      $s = preg_replace ("/^.*DEFAULT_USER.*$/m", 
                         "define('DEFAULT_USER', ". intval($_POST['defaultUserSelection']) .");", 
                         $s);

      if ($f = fopen ($configFile, 'w')) {

        fwrite($f, $s);
        fclose($f);

        echo '<p>Updated the configuration file with default user selection.</p>';
        echo '<p>Now you may <a href="auth.php">log in</a>, and edit your preferences to add a feed.</p>';

      } else {
        echo '<p>Could not write configuration file!</p>';
      }
    } else {
      echo '<p>Could not read configuration file!</p>';
    }
  } else {
    echo '<p>No changes made.</p>';
  }

  echo checkInsecureFilePerms();

  // -------------------------------------------------------------------------
  // Present initial screen: database settings
  //
} else {
  require_once('inc/config.php');
?>

  <form action="<?php echo $_SERVER['PHP_SELF'] ?>" method="POST">
    <fieldset>
      <legend>Install Rnews</legend>
      
      <p>This page installs Rnews version <?php echo RNEWS_VERSION ?>.  The latest version is always available at the <a href="http://rnews.sourceforge.net/">Rnews web site</a>.</p>
      <p>Requirements: 1) web server with PHP version &gt;= 4.x, and 2) mysql account.</p>
<?php
  if (!is_readable($configFile) || !is_writable($configFile)) {
    echo '<p class="error">Error: the file '. $configFile .' is not writable.  You will not be able to install until you change the file mode to make it writable.</p>';
  }
?>
      <p>Title to be shown on all pages next to the Rnews logo.<br />
      <input type="text" name="title" size="25" maxlength="100" value="<?php echo RNEWS_TITLE ?>" /></p>

      <fieldset>
        <legend>Database Information</legend>
        <p><em>Note: only MySQL is currently supported.</em></p>

        <ol>
        <li><p>Enter the <i>hostname</i> on which the database resides. To use the same host as the web server enter <i>localhost</i> (on a Unix derivative), or '<b>.</b>' (on Windows):<br />
        <input type="text" name="dbhost" size="50" maxlength="200" value="<?php echo DB_HOST ?>" /></p></li>

        <li><p>Enter the <i>user ID</i> for accessing the database:<br />
        <input type="text" name="dbuser" size="20" maxlength="32" value="<?php echo DB_USER ?>" /></p></li>

        <li><p>Enter the <i>password</i> for accessing the database:<br />
        <input type="password" name="dbpass" size="20" maxlength="32" value="<?php echo htmlspecialchars(DB_PASS) ?>" /></p></li>

        <li><p>Enter the <i>name of the database</i> to create or use:<br />
        <input type="text" name="dbname" size="20" maxlength="32" value="<?php echo DB_DATABASE ?>" /></p></li>

        <li><p>Enter the <i>table prefix</i>, if desired (useful if your DB provider only allows you one database):<br />
        <input type="text" name="dbprefix" size="20" maxlength="32" value="<?php echo DB_PREFIX ?>" /></p></li></ol>
      </fieldset>

      <fieldset>
        <legend>Security</legend>

        <ol>
        <li><p>To control who may create an account, define a secret below.  New users will have to enter the secret before being allowed to register an account.  This is highly recommended, and you should <b>change the default value</b>.</p>
        <p>New user secret:<br />
        <input type="text" name="secret" size="32" maxlength="32" value="<?php echo htmlspecialchars(NEWUSER_SECRET) ?>" /></p></li>

        <li><p>If you can use SSL on your web server, it is recommended to force logins and password changes to occur over this secure link.  This prevents passwords from traversing the network in the clear, subject to eavesdropping.</p>
        <p>Force secure login?
        <input type="checkbox" name="forceSlogin" <?php echo (FORCE_SLOGIN ? 'checked="true"' : '') ?> /> Yes</p></li>

        <li><p>Since RSS feeds are published by other websites and are displayed on a web page, you probably do not want to allow arbitrary HTML tags to appear there.  This opens you up to Cross-Site Scripting (XSS) attacks.  Select the types of tags you wish to <b>allow</b> below.</p>
        <p>Allowed tags:<br />
        <input type="checkbox" name="xssStruct" <?php echo ((XSS_TAGS & XSS_TAGS_STRUCT) ? 'checked="true"' : '') ?> /> Basic structure&nbsp;&nbsp;
        <input type="checkbox" name="xssFormat" <?php echo ((XSS_TAGS & XSS_TAGS_FORMAT) ? 'checked="true"' : '') ?> /> Formatting&nbsp;&nbsp;
        <input type="checkbox" name="xssImage" <?php echo ((XSS_TAGS & XSS_TAGS_IMAGE) ? 'checked="true"' : '') ?> /> Images&nbsp;&nbsp;
        <input type="checkbox" name="xssTable" <?php echo ((XSS_TAGS & XSS_TAGS_TABLE) ? 'checked="true"' : '') ?> /> Tables&nbsp;&nbsp;
        <input type="checkbox" name="xssAny" <?php echo (((XSS_TAGS & XSS_TAGS_ANY) == XSS_TAGS_ANY) ? 'checked="true"' : '') ?> /> Everything else, too (not recommended)</p></li>

        <li><p>By default, each user must log in to view their feeds, otherwise no feeds are visible.  However, you may wish to select one user's feeds to be shown to anyone at all without logging in.  Feeds can be added and deleted only after logging in, but anyone can mark them as read.</p>
        <p>Choose a default user after installation?
        <input type="checkbox" name="defaultUser" <?php echo ((DEFAULT_USER > 0) ? 'checked="true"' : '') ?> /> Yes</p></li>
        </ol>
      </fieldset>

      <fieldset>
        <legend>Feed Options</legend>

        <ol>
        <li><p>Feed contents can be cached on the web server.  This is only useful if it is likely that multiple users will subscribe to some of the same feeds, since Rnews only fetches feeds every so often anyway.</p>
        <p>Cache feeds?
        <input type="checkbox" name="cache" <?php echo (MAGPIE_CACHE_ON ? 'checked="true"' : '') ?> /> Yes</p></li>

        <li><p>Feed images can be copied to the web server to reduce load on the publishing sites.</p>
        <p>Cache feed images?
        <input type="checkbox" name="cacheImages" <?php echo (FEED_IMG_CACHE ? 'checked="true"' : '') ?> /> Yes</p></li>

        <li><p>If caching feed images, where should they be stored?  This folder name is relative to the installation folder.  It must be writable by the web server.<br />
        <input type="text" name="imgFolder" size="25" maxlength="150" value="<?php echo FEED_IMG_FOLDER ?>" /></p></li>

        <li><p>If feed images are cached (see above), they can also be first resized so they don't mess up the layout of Rnews too badly. Blank or 0 (zero) turns off resizing.</p>
        <p>Maximum image height (pixels):<br />
        <input type="text" name="imgHeight" size="5" maxlength="5" value="<?php echo FEED_IMG_HEIGHT ?>" /></p></li>
        </ol>
      </fieldset>

      <p><input type="Submit" name="writeConfig" value="Install Rnews"></p>
    </fieldset>
  </form>

<?php
}
echo "</div></div></div>";    // container, wrapper, category
include ('foot.php');
?>
